#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
check_talend_job
------------------------------
A nagios-like plugin to check if talend jobs executed successfully

"""

__docformat__ = 'restructuredtext en'

from .check_talend_job import main
# vim:set et sts=4 ts=4 tw=80:



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Check if talend job runned without problem
"""
from datetime import datetime
import configparser
import argparse
import glob
import sys
import os
import re


config = {}

def get_time(line):
    try:
        strTime = re.split(' |;', line)[1]
        return datetime.strptime(strTime, "%H:%M:%S")
    except (IndexError, ValueError):
        raise Exception("Wrong format for line of log file")


def get_latest_job(job):    # pragma: no cover
    try:
        path = config['basePath'] + job + '/*'
        list_of_files = glob.glob(path)
        latest = max(list_of_files, key=os.path.getctime)
        return latest
    except ValueError:
        raise Exception("Path %s non-existant" % path)


def check_log_file(pathToFile):
    try:
        with open(pathToFile) as logFile:
            lines = logFile.readlines()

        startTime = get_time(lines[0])
        endTime = get_time(lines[-2])
        elapsedTime = endTime - startTime

        if config['endingString'] in lines[-2]:
            message = "OK: task run in {0}s | time={1}".format(
                                                            elapsedTime.seconds,
                                                            elapsedTime.total_seconds())
            return message, 0
        else:
            message = "CRITICAL: {0} | time={1}".format(
                                                        lines[-2].strip(),
                                                        elapsedTime.total_seconds())
            return message, 2

    except IOError:
        return "UNKNOWN: File %s not found" % pathToFile, 3
    except IndexError:  # Too few lines
        return "UNKNOWN: Not a talend log file", 3
    except Exception as e:
        return "UNKNOWN: " + " ".join(e.args), 3


def parse_args():   # pragma: no cover
    argp = argparse.ArgumentParser(description=__doc__)
    argp.add_argument('-j', '--job', required=True, help='Name of job')
    return argp.parse_args()


def create_conf():
    cfg = configparser.ConfigParser()
    cfg.add_section('main')


    global config
    config['basePath'] = input("Enter path to all job logs (ex : 'C:\SomeDir\\'): ")
    config['endingString'] = input("Enter string ending a correct talend log (ex ';END\\n') : ")

    cfg.set('main', 'basePath', config['basePath'])
    cfg.set('main', 'endingString', config['endingString'])

    cfg.write(open(sys.prefix+'/config.ini', 'w'))


def configure():
    cfg = configparser.ConfigParser()
    cfg.read(sys.prefix+"/config.ini")
    try:
        global config
        config['basePath'] = cfg.get('main', 'basePath')
        config['endingString'] = cfg.get('main', 'basePath')
    except configparser.NoSectionError:
        create_conf()


def main():
    args = parse_args()
    configure()

    message, status = check_log_file(get_latest_job(args.job))
    print(message)
    exit(status)


if __name__ == '__main__':  # pragma: no cover
    main()



import unittest
import mock

import sys
from contextlib import contextmanager
from io import StringIO

from datetime import datetime

from check_talend_job import check_talend_job


@contextmanager
def captured_output():
    """ A context manager to capture output """
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err


class Test__get_time(unittest.TestCase):
    def test__good_format(self):
        time = datetime(1900,1,1,12,12,12)
        self.assertEqual(check_talend_job.get_time("string 12:12:12;string string"), time)

    def test__bad_time(self):
        with self.assertRaises(Exception):
            check_talend_job.get_time("string 12:12:61;string string")

    def test__bad_format(self):
        with self.assertRaises(Exception):
            check_talend_job.get_time("12:12:12")


class Test__check_log_file(unittest.TestCase):

    def mock_check_log_file(self, list_lines):
        check_talend_job.config = {"endingString": ";END\n"}
        mopen = mock.mock_open(read_data=list_lines)
        with mock.patch('check_talend_job.check_talend_job.open', mopen, create=True):
            return check_talend_job.check_log_file("/dev/null")


    def test__ok_file(self):
        ok_file = "".join(["String 10:10:10;string string\n",
                          "String 10:10:10;string string\n",
                          "String 10:10:11;END\n",
                          "\n"])
        ok_return = "OK: task run in 1s | time=1.0", 0

        self.assertEqual(self.mock_check_log_file(ok_file), ok_return)

    def test__critical_file(self):
        critical_file = "".join(["String 10:10:10;string string\n",
                          "String 10:10:10;string string\n",
                          "String 10:10:11;Exception string\n",
                          "\n"])
        critical_return = 'CRITICAL: String 10:10:11;Exception string | time=1.0', 2

        self.assertEqual(self.mock_check_log_file(critical_file), critical_return)

    def test__file_not_found(self):
        self.assertEqual(check_talend_job.check_log_file(''), ('UNKNOWN: File  not found', 3))

    def test__too_few_line(self):
        too_short_file = "String 10:10:10;string string\n"
        self.assertEqual(self.mock_check_log_file(too_short_file), ('UNKNOWN: Not a talend log file', 3))

    def test__bad_file_format(self):
        bad_file = "String"
        bad_result = "UNKNOWN: Wrong format for line of log file", 3
        self.assertEqual(self.mock_check_log_file(bad_file), bad_result)


class Test__main(unittest.TestCase):
    message = "message"

    @mock.patch("check_talend_job.check_talend_job.get_latest_job")
    @mock.patch("check_talend_job.check_talend_job.parse_args")
    @mock.patch("check_talend_job.check_talend_job.check_log_file")
    def test_output(self, check_log_file, parse_args, get_latest_job):
        check_log_file.return_value = self.message, 0
        args = mock.MagicMock()
        args.job = None
        parse_args.return_value = args
        get_latest_job.return_value = None
        with captured_output() as (out, err):
            with self.assertRaises(SystemExit) as exitException:
                check_talend_job.main()
        output = out.getvalue().strip()
        self.assertEqual(output, self.message)
        self.assertEqual(exitException.exception.code, 0)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os


version = '0.2-dev'

here = os.path.abspath(os.path.dirname(__file__))

def read_file(*pathes):
    path = os.path.join(here, *pathes)
    if os.path.isfile(path):
        with open(path, 'r') as desc_file:
            return desc_file.read()
    else:
        return ''

desc_files = (('README.md',), ('docs', 'CHANGES.rst'),
                ('docs', 'CONTRIBUTORS.rst'))

long_description = '\n\n'.join([read_file(*pathes) for pathes in desc_files])

install_requires=['setuptools']


setup(name='check_talend_job',
      version=version,
      description="A nagios-like plugin to check if talend jobs executed successfully",
      long_description=long_description,
      platforms = ["any"],
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        ],
      keywords="nagios",
      author="Elio Maisonneuve",
      author_email="maisonneuv@eisti.eu",
      url="https://github.com/paulla/check_talend_job",
      license="BSD",
      packages=find_packages("src"),
      package_dir = {"": "src"},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      check_talend_job = check_talend_job:main
      """,
      )

# vim:set et sts=4 ts=4 tw=80:
